@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013, Georg Brandl.  All rights reserved.
+# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -10,5 +10,10 @@
 
 __all__ = ['start_fitter', 'start_loader']
 
+from ufit.gui.loggers import getLogger
+logger = getLogger('ufit')
+
 from ufit.gui.fitter import start as start_fitter
 from ufit.gui.dataloader import start as start_loader
+
+import ufit.guiresource  # register icon resources
