@@ -2,46 +2,45 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013, Georg Brandl.  All rights reserved.
+# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
 """Data operations panel."""
 
-from numpy import sqrt, mean, array, ones
+from numpy import sqrt, ones
 
 from PyQt4.QtCore import pyqtSignature as qtsig, SIGNAL
-from PyQt4.QtGui import QWidget, QDialog, QListWidgetItem, QMessageBox
+from PyQt4.QtGui import QWidget, QDialog, QMessageBox
 
+from ufit.data.merge import rebin, floatmerge
 from ufit.gui.common import loadUi
-from ufit.gui.mapping import MappingWindow
-from ufit.data.merge import rebin
-from ufit.data.dataset import Dataset
+from ufit.gui.session import session
 
 
 class DataOps(QWidget):
 
-    def __init__(self, parent, panellist):
+    def __init__(self, parent):
         QWidget.__init__(self, parent)
+        self.item = None
         self.data = None
         self.picking = None
         self.picked_points = []
-        self.panellist = panellist
 
         loadUi(self, 'dataops.ui')
-        self.pickedlabel.hide()
+        self.pickedLbl.hide()
 
-    def initialize(self, data, model):
-        self.data = data
-        self.model = model
+    def initialize(self, item):
+        self.data = item.data
+        self.model = item.model
+        self.item = item
         if self.data.fitmin is not None:
-            self.limitmin.setText('%.5g' % self.data.fitmin)
+            self.limitminEdit.setText('%.5g' % self.data.fitmin)
         if self.data.fitmax is not None:
-            self.limitmax.setText('%.5g' % self.data.fitmax)
-        self.monscale.setText(str(self.data.nscale))
-        # XXX fix this mess (title, name, environment, sources, ...)
-        self.datatitle.setText(self.data.meta.get('title', ''))
-        self.nameEdit.setText(self.data.name or '')
+            self.limitmaxEdit.setText('%.5g' % self.data.fitmax)
+        self.monscaleEdit.setText(str(self.data.nscale))
+        self.titleEdit.setText(self.data.title)
+        self.nameEdit.setText(self.data.name)
 
     def on_canvas_pick(self, event):
         if not hasattr(event, 'artist'):
@@ -50,7 +49,7 @@ class DataOps(QWidget):
             xdata = event.artist.get_xdata()[event.ind]
             ydata = event.artist.get_ydata()[event.ind]
             self.picked_points.append(xdata)
-            self.pickedlabel.setText('%d picked' % len(self.picked_points))
+            self.pickedLbl.setText('%d picked' % len(self.picked_points))
             event.canvas.figure.gca().plot([xdata], [ydata], 'ow', ms=8,
                                            mec='blue')
             event.canvas.draw()
@@ -58,14 +57,14 @@ class DataOps(QWidget):
     @qtsig('')
     def on_badResetBtn_clicked(self):
         self.data.reset_mask()
-        self.emit(SIGNAL('replotRequest'))
-        self.emit(SIGNAL('dirty'))
+        self.emit(SIGNAL('replotRequest'), None)
+        session.set_dirty()
 
     @qtsig('')
     def on_badPointsBtn_clicked(self):
         if self.picking == 'bad':
             self.badPointsBtn.setText('Start')
-            self.pickedlabel.hide()
+            self.pickedLbl.hide()
             self.picking = None
             self.removeBadPoints(self.picked_points)
         elif not self.picking:
@@ -74,52 +73,71 @@ class DataOps(QWidget):
             self.emit(SIGNAL('pickRequest'), self)
             self.picking = 'bad'
             self.picked_points = []
-            self.pickedlabel.setText('0 picked')
-            self.pickedlabel.show()
+            self.pickedLbl.setText('0 picked')
+            self.pickedLbl.show()
 
     @qtsig('')
     def on_limitsBtn_clicked(self):
         try:
-            limitmin = float(self.limitmin.text())
+            limitmin = float(self.limitminEdit.text())
         except ValueError:
             limitmin = None
         try:
-            limitmax = float(self.limitmax.text())
+            limitmax = float(self.limitmaxEdit.text())
         except ValueError:
             limitmax = None
         self.data.fitmin, self.data.fitmax = limitmin, limitmax
-        self.emit(SIGNAL('replotRequest'))
-        self.emit(SIGNAL('dirty'))
+        self.emit(SIGNAL('replotRequest'), None)
+        session.set_dirty()
 
     def removeBadPoints(self, points):
         """'Remove' bad data points (just mask them out)."""
         for point in points:
             self.data.mask[self.data.x == point] = False
-        self.emit(SIGNAL('replotRequest'))
-        self.emit(SIGNAL('dirty'))
+        self.emit(SIGNAL('replotRequest'), None)
+        session.set_dirty()
 
     @qtsig('')
     def on_rebinBtn_clicked(self):
-        binsize = self.precision.value()
+        try:
+            binsize = float(self.precisionEdit.text())
+        except ValueError:
+            QMessageBox.warning(self, 'Error', 'Enter a valid precision.')
+            return
         new_array = rebin(self.data._data, binsize)
         self.data.__init__(self.data.meta, new_array,
                            self.data.xcol, self.data.ycol, self.data.ncol,
                            self.data.nscale, name=self.data.name,
                            sources=self.data.sources)
         self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
+        session.set_dirty()
+
+    @qtsig('')
+    def on_floatmergeBtn_clicked(self):
+        try:
+            binsize = float(self.precisionEdit.text())
+        except ValueError:
+            QMessageBox.warning(self, 'Error', 'Enter a valid precision.')
+            return
+        new_array = floatmerge(self.data._data, binsize)
+        self.data.__init__(self.data.meta, new_array,
+                           self.data.xcol, self.data.ycol, self.data.ncol,
+                           self.data.nscale, name=self.data.name,
+                           sources=self.data.sources)
+        self.emit(SIGNAL('replotRequest'), None)
+        session.set_dirty()
 
     @qtsig('')
     def on_cloneBtn_clicked(self):
         new_data = self.data.copy()
         new_model = self.model.copy()
-        self.emit(SIGNAL('newData'), new_data, True, new_model)
-        self.emit(SIGNAL('dirty'))
+        from ufit.gui.scanitem import ScanDataItem
+        session.add_item(ScanDataItem(new_data, new_model), self.item.group)
 
     @qtsig('')
     def on_mulBtn_clicked(self):
         try:
-            const = float(self.mul_constant.text())
+            const = float(self.scaleConstEdit.text())
         except ValueError:
             return
         self.data.y *= const
@@ -127,33 +145,43 @@ class DataOps(QWidget):
         self.data.dy *= const
         self.data.dy_raw *= const
         self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
+        session.set_dirty()
 
     @qtsig('')
     def on_addBtn_clicked(self):
         try:
-            const = float(self.add_constant.text())
+            const = float(self.addConstEdit.text())
         except ValueError:
             return
         self.data.y += const
         self.data.y_raw += const * self.data.norm
         self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
+        session.set_dirty()
+
+    @qtsig('')
+    def on_scaleXBtn_clicked(self):
+        try:
+            const = float(self.scaleXConstEdit.text())
+        except ValueError:
+            return
+        self.data.x *= const
+        self.emit(SIGNAL('replotRequest'), None)
+        session.set_dirty()
 
     @qtsig('')
     def on_shiftBtn_clicked(self):
         try:
-            const = float(self.shift_constant.text())
+            const = float(self.shiftConstEdit.text())
         except ValueError:
             return
         self.data.x += const
         self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
+        session.set_dirty()
 
     @qtsig('')
     def on_monscaleBtn_clicked(self):
         try:
-            const = int(self.monscale.text())
+            const = int(self.monscaleEdit.text())
         except ValueError:
             return
         self.data.nscale = const
@@ -162,38 +190,38 @@ class DataOps(QWidget):
         self.data.dy = sqrt(self.data.y_raw)/self.data.norm
         self.data.yaxis = self.data.ycol + ' / %s %s' % (const, self.data.ncol)
         self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
+        session.set_dirty()
 
     @qtsig('')
     def on_titleBtn_clicked(self):
-        self.data.meta.title = str(self.datatitle.text())
+        self.data.meta.title = str(self.titleEdit.text())
         self.emit(SIGNAL('titleChanged'))
-        self.emit(SIGNAL('dirty'))
+        session.set_dirty()
         self.emit(SIGNAL('replotRequest'), None)
 
     @qtsig('')
     def on_nameBtn_clicked(self):
         self.data.name = str(self.nameEdit.text())
-        self.emit(SIGNAL('dirty'))
+        session.set_dirty()
         self.emit(SIGNAL('replotRequest'), None)
 
     @qtsig('')
     def on_subtractBtn_clicked(self):
+        from ufit.gui.scanitem import ScanDataItem
         dlg = QDialog(self)
         loadUi(dlg, 'subtract.ui')
-        for i, p in enumerate(self.panellist):
-            QListWidgetItem('%d' % p.index, dlg.setList, i)
+        data2obj = dlg.setList.populate(ScanDataItem)
         if dlg.exec_() != QDialog.Accepted:
             return
-        items = dlg.setList.selectedItems()
-        if not items:
+        witems = dlg.setList.selectedItems()
+        if not witems:
             return
         try:
             prec = float(dlg.precisionEdit.text())
         except ValueError:
             QMessageBox.warning(self, 'Error', 'Please enter a valid precision.')
             return
-        bkgd_data = self.panellist[items[0].type()].data
+        bkgd_data = data2obj[witems[0].type()].data
         if not dlg.destructBox.isChecked():
             new_data = self.data.copy()
         else:
@@ -222,178 +250,8 @@ class DataOps(QWidget):
 
         if not dlg.destructBox.isChecked():
             new_model = self.model.copy()
-            self.emit(SIGNAL('newData'), new_data, True, new_model)
+            from ufit.gui.scanitem import ScanDataItem
+            session.add_item(ScanDataItem(new_data, new_model), self.item.group)
         else:
             self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
-
-
-class MultiDataOps(QWidget):
-
-    def __init__(self, parent):
-        QWidget.__init__(self, parent)
-        self.data = None
-
-        loadUi(self, 'multiops.ui')
-
-    def initialize(self, panels):
-        self.panels = panels
-        self.datas = [p.data for p in panels]
-        self.monscale.setText(str(int(mean([d.nscale for d in self.datas]))))
-        self.onemodel.clear()
-        self.onemodel.addItems(['%d' % p.index for p in panels])
-
-    @qtsig('')
-    def on_rebinBtn_clicked(self):
-        binsize = self.precision.value()
-        for data in self.datas:
-            new_array = rebin(data._data, binsize)
-            data.__init__(data.meta, new_array,
-                          data.xcol, data.ycol, data.ncol,
-                          data.nscale, name=data.name,
-                          sources=data.sources)
-        self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
-
-    @qtsig('')
-    def on_mulBtn_clicked(self):
-        try:
-            const = float(self.mul_constant.text())
-        except ValueError:
-            return
-        for data in self.datas:
-            data.y *= const
-            data.y_raw *= const
-            data.dy *= const
-            data.dy_raw *= const
-        self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
-
-    @qtsig('')
-    def on_addBtn_clicked(self):
-        try:
-            const = float(self.add_constant.text())
-        except ValueError:
-            return
-        for data in self.datas:
-            data.y += const
-            data.y_raw += const * data.norm
-        self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
-
-    @qtsig('')
-    def on_shiftBtn_clicked(self):
-        try:
-            const = float(self.shift_constant.text())
-        except ValueError:
-            return
-        for data in self.datas:
-            data.x += const
-        self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
-
-    @qtsig('')
-    def on_monscaleBtn_clicked(self):
-        try:
-            const = int(self.monscale.text())
-        except ValueError:
-            return
-        for data in self.datas:
-            data.nscale = const
-            data.norm = data.norm_raw / const
-            data.y = data.y_raw/data.norm
-            data.dy = sqrt(data.y_raw)/data.norm
-        self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
-
-    @qtsig('')
-    def on_mergeBtn_clicked(self):
-        precision = self.mergeprecision.value()
-        new_data = self.datas[0].merge(precision, *self.datas[1:])
-        self.emit(SIGNAL('newData'), new_data)
-
-    @qtsig('')
-    def on_onemodelBtn_clicked(self):
-        which = self.onemodel.currentIndex()
-        if which < 0:
-            return
-        model = self.panels[which].model
-        for i, panel in enumerate(self.panels):
-            if i == which:
-                continue
-            panel.handle_new_model(model.copy(), keep_paramvalues=False)
-        self.emit(SIGNAL('replotRequest'), None)
-        self.emit(SIGNAL('dirty'))
-
-    @qtsig('')
-    def on_paramsetBtn_clicked(self):
-        dlg = ParamSetDialog(self, self.panels)
-        if dlg.exec_() != QDialog.Accepted:
-            return
-        self.emit(SIGNAL('newData'), dlg.new_data)
-
-    @qtsig('')
-    def on_mappingBtn_clicked(self):
-        wnd = MappingWindow(self)
-        wnd.set_datas([panel.data for panel in self.panels])
-        wnd.show()
-
-    @qtsig('')
-    def on_globalfitBtn_clicked(self):
-        QMessageBox.warning(self, 'Sorry', 'Not implemented yet.')
-
-
-class ParamSetDialog(QDialog):
-    def __init__(self, parent, panels):
-        QDialog.__init__(self, parent)
-        loadUi(self, 'paramset.ui')
-        self.new_data = None
-        self.panels = panels
-
-        allvalues = set()
-        for panel in panels:
-            if not panel.model or not panel.data:
-                return
-            values = set([p.name + ' (parameter)' for p in panel.model.params] +
-                         [mname + ' (from data)' for mname in panel.data.meta if
-                          not mname.startswith('col_')])
-            if not allvalues:
-                allvalues = values
-            else:
-                allvalues &= values
-
-        allvalues = sorted(allvalues)
-        self.xvalueBox.addItems(allvalues)
-        self.yvalueBox.addItems(allvalues)
-
-    def exec_(self):
-        res = QDialog.exec_(self)
-        if res != QDialog.Accepted:
-            return res
-        xx, yy, dy = [], [], []
-        xp = yp = False
-        xv = str(self.xvalueBox.currentText())
-        if xv.endswith(' (parameter)'):
-            xp = True
-        xv = xv[:-12]
-        yv = str(self.yvalueBox.currentText())
-        if yv.endswith(' (parameter)'):
-            yp = True
-        yv = yv[:-12]
-
-        for panel in self.panels:
-            if xp:
-                xx.append(panel.model.paramdict[xv].value)
-            else:
-                xx.append(panel.data.meta[xv])
-            if yp:
-                yy.append(panel.model.paramdict[yv].value)
-                dy.append(panel.model.paramdict[yv].error)
-            else:
-                yy.append(panel.data.meta[yv])
-                dy.append(1)
-        xx, yy, dy = map(array, [xx, yy, dy])
-
-        self.new_data = Dataset.from_arrays(str(self.nameBox.text()),
-                                            xx, yy, dy, xcol=xv, ycol=yv)
-        return res
+            session.set_dirty()
