@@ -2,7 +2,7 @@
 # *****************************************************************************
 # ufit, a universal scattering fitting suite
 #
-# Copyright (c) 2013, Georg Brandl.  All rights reserved.
+# Copyright (c) 2013-2014, Georg Brandl and contributors.  All rights reserved.
 # Licensed under a 2-clause BSD license, see LICENSE.
 # *****************************************************************************
 
@@ -12,11 +12,12 @@ neutrons.instruments.tas.tasres from the neutrons Python package, compiled by
 Marc Janoschek.
 """
 
+import os
 import multiprocessing
 
 from numpy import pi, radians, degrees, sin, cos, tan, arcsin, arccos, \
      arctan2, abs, sqrt, real, matrix, diag, cross, dot, array, arange, \
-     zeros, concatenate, reshape, delete, exp
+     zeros, concatenate, reshape, delete, loadtxt
 from numpy.random import randn
 from numpy.linalg import inv, det, eig, norm
 
@@ -214,8 +215,88 @@ class unitcell(object):
         return degrees(arcsin(Q*lam/4/pi))
 
 
-# instrumental setup description
+# parameters
+PARNAMES = [
+    'dm',
+    'da',
+    'etam',
+    'etas',
+    'etaa',
+    'sm',
+    'ss',
+    'sa',
+    'k',
+    'kfix',
+    'alpha1',
+    'alpha2',
+    'alpha3',
+    'alpha4',
+    'beta1',
+    'beta2',
+    'beta3',
+    'beta4',
+    'as',
+    'bs',
+    'cs',
+    'aa',
+    'bb',
+    'cc',
+    'ax',
+    'ay',
+    'az',
+    'bx',
+    'by',
+    'bz',
+    # 'qx',    # these are irrelevant for our resolution calc.
+    # 'qy',
+    # 'qz',
+    # 'en',
+    # 'dqx',
+    # 'dqy',
+    # 'dqz',
+    # 'de',
+    # 'gh',
+    # 'gk',
+    # 'gl',
+    # 'gmod',
+]
+
 CFGNAMES = [
+    'sourcetype',
+    'sourcewidth',
+    'sourceheight',
+    'guideon',
+    'guidedivh',
+    'guidedivv',
+    'samplecuboid',
+    'samplewidthperp',
+    'samplewidthpar',
+    'sampleheight',
+    'detrectangular',
+    'detwidth',
+    'detheight',
+    'monothickness',
+    'monowidth',
+    'monoheight',
+    'anathickness',
+    'anawidth',
+    'anaheight',
+    'distsm',
+    'distms',
+    'distsa',
+    'distad',
+    'curvmonoh',
+    'curvmonov',
+    'curvanah',
+    'curvanav',
+    'distmoni',
+    'moniwidth',
+    'moniheight',
+]
+
+
+# instrumental setup description
+CFGDESC = [
     "=0 for circular source, =1 for rectangular source",
     "width/diameter of the source (cm)",
     "height/diameter of the source (cm)",
@@ -261,6 +342,7 @@ class resmat(object):
 
     This code was ported from Bertrand Roessli's octave version of the ILL
     rescal MATLAB tools.
+    Checked against rescal5 rc_popma.m from ILL matlab libraries from 06/2013.
     """
 
     def __init__(self, cfg, par):
@@ -269,6 +351,9 @@ class resmat(object):
         # experimental setup (plus easily changed instrument things)
         self.par = par.copy()
 
+        # use a fixed resolution matrix
+        self.fixed_res = False
+
         # a string message or None if calculations are fine
         self.ERROR = None
 
@@ -290,7 +375,9 @@ class resmat(object):
         self._cache = {}
 
     def calc_popovici(self):
-        """Performs the actual calculation."""
+        """Performs the actual calculation.
+
+        """
         # reset errors before new calculation
         self.ERROR = None
 
@@ -348,7 +435,6 @@ class resmat(object):
         ymon = self.cfg[14]               # width of monochromator [cm].
         zmon = self.cfg[15]               # height of monochromator [cm].
 
-
         xana = self.cfg[16]               # thickness of analyser [cm].
         yana = self.cfg[17]               # width of analyser [cm].
         zana = self.cfg[18]               # height of analyser [cm].
@@ -358,10 +444,7 @@ class resmat(object):
         L2 = self.cfg[21]                 # distance between sample and analyser [cm].
         L3 = self.cfg[22]                 # distance between analyser and detector [cm].
 
-        romh = sm*self.cfg[23]            # horizontal curvature of monochromator 1/radius [cm-1].
-        romv = sm*self.cfg[24]            # vertical curvature of monochromator [cm-1].
-        roah = sa*self.cfg[25]            # horizontal curvature of analyser [cm-1].
-        roav = sa*self.cfg[26]            # vertical curvature of analyser [cm-1].
+        # curvature radii are handled below, after we calculated thetam and thetaa.
 
         L1mon = self.cfg[27]              #distance monochromator monitor [cm]
         monitorw = self.cfg[28]/sqrt(12)  #monitor width [cm]
@@ -372,26 +455,52 @@ class resmat(object):
 
         # _____________________________________________________________________________
 
-        # In addition the parameters f, energy pre-multiplier-f*w
-        # where f=0.48 for meV to ang-2 - and q0 which is the wavevector
-        # transfer in ang-1, are passed over.
+        # THIS ASSUMES w IS IN meV!
 
         # Calculate ki and kf, thetam and thetaa
-        ki = abs(sqrt(kfix**2+(fx-1) * MEV2AA2 * w))  # kinematical equations.
-        kf = abs(sqrt(kfix**2-(2-fx) * MEV2AA2 * w))
+        ki = abs(sqrt(kfix**2 + (fx-1) * MEV2AA2 * w))  # kinematical equations.
+        kf = abs(sqrt(kfix**2 - (2-fx) * MEV2AA2 * w))
 
         # Test if scattering triangle is closed
 
-        cos_2theta = (ki**2 + kf**2 - q0**2)/(2*ki*kf)
+        cos_2theta = (ki**2 + kf**2 - q0**2) / (2*ki*kf)
         # print cos_2theta
-        if (cos_2theta > 1 or cos_2theta < -1):
+        if not -1 <= cos_2theta <= 1:
             self.ERROR = 'scattering triangle not closed'
             return
 
         thetaa = sa*arcsin(pi/(da*kf))      # theta angles for analyser
         thetam = sm*arcsin(pi/(dm*ki))      # and monochromator.
-        thetas = ss*0.5*arccos((ki**2+kf**2-q0**2)/(2*ki*kf)) # scattering angle from sample.
-        phi = arctan2(-kf*sin(2*thetas),ki-kf*cos(2*thetas))
+        thetas = ss*0.5*arccos((ki**2 + kf**2 - q0**2) / (2*ki*kf)) # scattering angle from sample.
+        phi = arctan2(-kf*sin(2*thetas), ki - kf*cos(2*thetas))
+
+        # _____________________________________________________________________________
+
+        # automatic determination of curvature, added GB 06/2013 (not in RESCAL5)
+        if self.cfg[23] == -1:
+            if flag_guide:
+                romh = sm*0.5*(1./L1)*sin(abs(thetam))
+            else:
+                romh = sm*0.5*(1./L0 + 1./L1)*sin(abs(thetam))
+        else:
+            romh = sm*self.cfg[23]          # horizontal curvature of monochromator 1/radius [cm-1].
+        if self.cfg[24] == -1:
+            if flag_guide:
+                romv = sm*0.5*(1./L1)*sin(abs(thetam))**(-1)
+            else:
+                romv = sm*0.5*(1./L0 + 1./L1)*sin(abs(thetam))**(-1)
+        else:
+            romv = sm*self.cfg[24]          # vertical curvature of monochromator [cm-1].
+        if self.cfg[25] == -1:
+            roah = sa*0.5*(1./L2 + 1./L3)*sin(abs(thetaa))
+        else:
+            roah = sa*self.cfg[25]          # horizontal curvature of analyser [cm-1].
+        if self.cfg[26] == -1:
+            roav = sa*0.5*(1./L2 + 1./L3)*sin(abs(thetaa))**(-1)
+        else:
+            roav = sa*self.cfg[26]          # vertical curvature of analyser [cm-1].
+
+        # _____________________________________________________________________________
 
         # Make up the matrices in appendix 1 of M. Popovici (1975).
 
@@ -401,10 +510,10 @@ class resmat(object):
             alf0_guide = MIN2RAD*2*pi*guide_h/ki
             bet0 = MIN2RAD*2*pi*guide_v/ki
 
-            if alf0_guide <= alf0:
-                alf0 = alf0_guide  # take into account collimator in the guide
+            if alf0_guide <= alf0:  # take into account collimator in the guide
+                alf0 = alf0_guide   # (not in RESCAL5)
 
-        G = matrix(zeros((8,8)))
+        G = matrix(zeros((8, 8)))
         G[0,0] = 1/alf0**2    # horizontal and vertical collimation matrix. The 4 Soller collimators
         G[1,1] = 1/alf1**2    # are described by a 8x8 diagonal matrix with non-zero elements.
         G[2,2] = 1/bet0**2    # alfi and beti are the FWHM horizontal and vertical beam divergences in
@@ -414,13 +523,13 @@ class resmat(object):
         G[6,6] = 1/bet2**2
         G[7,7] = 1/bet3**2
 
-        F = matrix(zeros((4,4)))
+        F = matrix(zeros((4, 4)))
         F[0,0] = 1/etam**2    # monochromator and analyser mosaic matrix. horizontal and vertical mosaic
         F[1,1] = 1/etamp**2   # spreads for monochromator and analyzer crystals
         F[2,2] = 1/etaa**2
         F[3,3] = 1/etaap**2
 
-        C = matrix(zeros((4,8)))
+        C = matrix(zeros((4, 8)))
         C[0,0] = 1./2
         C[0,1] = C[0,0]
         C[2,4] = C[0,0]
@@ -432,7 +541,7 @@ class resmat(object):
         C[3,7] = -C[3,6]              # thetaa = -arcsin(Ta/2kf))epsilona
                                       # epsilonm/a = 1 if sample scattering
                                       # direction opposite do mono/ana scattering dir, -1 otherwise
-        A = matrix(zeros((6,8)))
+        A = matrix(zeros((6, 8)))
         A[0,0] = ki/(tan(thetam)*2)
         A[0,1] = -A[0,0]
         A[1,1] = ki
@@ -442,7 +551,7 @@ class resmat(object):
         A[4,4] = kf
         A[5,6] = A[4,4]
 
-        B = matrix(zeros((4,6)))
+        B = matrix(zeros((4, 6)))
         B[0,0] = cos(phi)
         B[0,1] = sin(phi)
         B[0,3] = -cos(phi-2*thetas)    # thetas = scattering angle of
@@ -458,14 +567,15 @@ class resmat(object):
 
         # Now include the spatial effects.
 
-        S1I = matrix(zeros((3,3)))
         # --- monochromator spatial covariances ----------
+        S1I = matrix(zeros((3, 3)))
         factor = f12
         S1I[0,0] = factor*xmon**2
         S1I[1,1] = factor*ymon**2
         S1I[2,2] = factor*zmon**2
-        S2I = zeros((3,3))
+
         # --- sample spatial covariances -----------
+        S2I = zeros((3, 3))
         if nsam == 0:
             factor = f16
         else:
@@ -474,14 +584,15 @@ class resmat(object):
         S2I[0,0] = factor*xsam**2
         S2I[1,1] = factor*ysam**2
         S2I[2,2] = f12*zsam**2
+
         # --- analyser spatial covariances ---------
         factor = f12
-        S3I = zeros((3,3))
+        S3I = zeros((3, 3))
         S3I[0,0] = factor*xana**2
         S3I[1,1] = factor*yana**2
         S3I[2,2] = factor*zana**2
         # --- construct the full spatial covariance matrix
-        SI = zeros((13,13))
+        SI = zeros((13, 13))
         # --- source covariance --------------------
         if nsou == 0:
             factor = f16      # factor for converting diameter**2 to variance**2
@@ -503,27 +614,27 @@ class resmat(object):
         SI[11,11] = factor*ydet**2
         SI[12,12] = factor*zdet**2
 
-        S = inv(5.545*SI)
+        S = inv(5.545 * SI)
 
-        T = matrix(zeros((4,13)))
-        T[0,0] = -1/(2*L0)
-        T[0,2] = cos(thetam)*(1/L1-1/L0)/2
-        T[0,3] = sin(thetam)*(1/L0+1/L1-2*romh/(sin(thetam)))/2
-        T[0,5] = sin(thetas)/(2*L1)
-        T[0,6] = cos(thetas)/(2*L1)
-        T[1,1] = -1/(2*L0*sin(thetam))
-        T[1,4] = (1/L0+1/L1-2*sin(thetam)*romv)/(2*sin(thetam))
-        T[1,7] = -1/(2*L1*sin(thetam))
-        T[2,5] = sin(thetas)/(2*L2)
-        T[2,6] = -cos(thetas)/(2*L2)
-        T[2,8] = cos(thetaa)*(1/L3-1/L2)/2
-        T[2,9] = sin(thetaa)*(1/L2+1/L3-2*roah/(sin(thetaa)))/2
-        T[2,11] = 1/(2*L3)
-        T[3,7] = -1/(2*L2*sin(thetaa))
-        T[3,10] = (1/L2+1/L3-2*sin(thetaa)*roav)/(2*sin(thetaa))
-        T[3,12] = -1/(2*L3*sin(thetaa))
+        T = matrix(zeros((4, 13)))
+        T[0,0] = -1./(2*L0)
+        T[0,2] = cos(thetam)*(1./L1 - 1./L0)/2
+        T[0,3] = sin(thetam)*(1./L0 + 1./L1 - 2*romh/sin(thetam))/2
+        T[0,5] = sin(thetas)/(2.*L1)
+        T[0,6] = cos(thetas)/(2.*L1)
+        T[1,1] = -1./(2*L0*sin(thetam))
+        T[1,4] = (1./L0 + 1./L1 - 2*sin(thetam)*romv)/(2*sin(thetam))
+        T[1,7] = -1./(2*L1*sin(thetam))
+        T[2,5] = sin(thetas)/(2.*L2)
+        T[2,6] = -cos(thetas)/(2.*L2)
+        T[2,8] = cos(thetaa)*(1./L3 - 1./L2)/2
+        T[2,9] = sin(thetaa)*(1./L2 + 1./L3 - 2*roah/sin(thetaa))/2
+        T[2,11] = 1./(2*L3)
+        T[3,7] = -1./(2*L2*sin(thetaa))
+        T[3,10] = (1./L2 + 1./L3 - 2*sin(thetaa)*roav)/(2*sin(thetaa))
+        T[3,12] = -1./(2*L3*sin(thetaa))
 
-        D = matrix(zeros((8,13)))
+        D = matrix(zeros((8, 13)))
         D[0,0] = -1./L0
         D[0,2] = -cos(thetam)/L0
         D[0,3] = sin(thetam)/L0
@@ -549,12 +660,12 @@ class resmat(object):
 
         # Construction of the resolution matrix
         #if det(D*inv(S+T.transpose()*F*T)*D.transpose()) != 0:
-        MI = B*A*(inv(inv(D*(inv(S+T.transpose()*F*T))*D.transpose())+G))*A.transpose()*B.transpose() # including spatial effects.
+        MI = B*A*inv(inv(D*inv(S + T.T*F*T)*D.T) + G)*A.T*B.T # including spatial effects.
         #else:
         #MI = B*A*(inv(G))*A.transpose()*B.transpose()
         #MI = B*A*[inv[G+C'*F*C]]*A'*B'  # Cooper and Nathans matrix,
-        MI[1,1] = MI[1,1]+q0**2*etas**2
-        MI[2,2] = MI[2,2]+q0**2*etasp**2
+        MI[1,1] = MI[1,1] + q0**2*etas**2
+        MI[2,2] = MI[2,2] + q0**2*etasp**2
         M = inv(MI)
         NP = 5.545*M                        # Correction factor as input parameters
         N = NP
@@ -564,9 +675,6 @@ class resmat(object):
         #mon = 1          # monochromator reflectivity
         #ana = 1          # detector and analyser crystal efficiency function. [const.]
 
-        self.vi = 0
-        self.vf = 0
-
         #if mon_flag==1
         #   vi = 1
         #else
@@ -588,13 +696,12 @@ class resmat(object):
         Rm = ki**3/tan(thetam)
         Ra = kf**3/tan(thetaa)
         R0 = Rm*Ra*(2*pi)**4/(64*pi**2*sin(thetam)*sin(thetaa))* \
-            sqrt(det(F)/det((D*(S+T.transpose()*F*T)**(-1)*D.transpose())**(-1)+G)) #Popovici
+            sqrt(det(F) / det(inv(D*inv(S + T.T*F*T)*D.T) + G)) #Popovici
         # Werner and Pynn correction
         # for mosaic spread of crystal.
-        R0 = abs(R0/(etas*sqrt(1/etas**2+q0**2*N[1,1])))
+        R0 = abs(R0 / (etas*sqrt(1/etas**2 + q0**2*N[1,1])))
 
-        #Transform prefactor to Chesser-Axe normalization
-        RM_ = matrix(zeros((4,4)))
+        RM_ = matrix(zeros((4, 4)))
         RM_[0,0] = M[0,0]
         RM_[1,0] = M[1,0]
         RM_[0,1] = M[0,1]
@@ -609,10 +716,14 @@ class resmat(object):
         RM_[3,3] = M[2,2]
         RM_[3,1] = M[2,1]
         RM_[1,3] = M[1,2]
+
+        #Transform prefactor to Chesser-Axe normalization
         R0 = R0/(2*pi)**2*sqrt(det(RM_))
+
         #---------------------------------------------------------------------------------------------
         #Include kf/ki part of cross section
         R0 = R0*kf/ki
+
         #include monitor efficiency
         #Normalisation to flux monitor
         #bshape = matrix(diag((ysrc,zsrc)))
@@ -622,16 +733,16 @@ class resmat(object):
         f = F[0:2][:,0:2]
         #c = C[0:2][:,0:4]
         t = matrix(zeros((2,7)))
-        t[0,0] = -1/(2*L0)  #mistake in paper
-        t[0,2] = cos(thetam)*(1/L1mon-1/L0)/2
-        t[0,3] = sin(thetam)*(1/L0+1/L1mon-2*romh/(sin(thetam)))/2
-        t[0,6] = 1/(2*L1mon)
-        t[1,1] = -1/(2*L0*sin(thetam))
-        t[1,4] = (1/L0+1/L1mon-2*sin(thetam)*romv)/(2*sin(thetam))
-        sinv = matrix(diag((ysrc,zsrc,xmon,ymon,zmon,monitorw,monitorh))) #S-1 matrix
+        t[0,0] = -1./(2*L0)  #mistake in paper
+        t[0,2] = cos(thetam)*(1./L1mon - 1./L0)/2
+        t[0,3] = sin(thetam)*(1./L0 + 1./L1mon - 2*romh/(sin(thetam)))/2
+        t[0,6] = 1./(2*L1mon)
+        t[1,1] = -1./(2*L0*sin(thetam))
+        t[1,4] = (1./L0 + 1./L1mon - 2*sin(thetam)*romv)/(2*sin(thetam))
+        sinv = matrix(diag((ysrc, zsrc, xmon, ymon, zmon, monitorw, monitorh))) #S-1 matrix
         s = sinv**(-1)
-        d = matrix(zeros((4,7)))
-        d[0,0] = -1/L0
+        d = matrix(zeros((4, 7)))
+        d[0,0] = -1./L0
         d[0,2] = -cos(thetam)/L0
         d[0,3] = sin(thetam)/L0
         d[2,1] = D[0,0]
@@ -639,13 +750,13 @@ class resmat(object):
         d[1,2] = cos(thetam)/L1mon
         d[1,3] = sin(thetam)/L1mon
         d[1,5] = 0
-        d[1,6] = 1/L1mon
-        d[3,4] = -1/L1mon
-        Rmon = (2*pi)**2/(8*pi*sin(thetam))*sqrt(det(f)/det((d*(s+t.transpose()*f*t)**(-1)*d.transpose())**(-1)+g))
-        Rmon = Rmon*ki**3/tan(thetam)
+        d[1,6] = 1./L1mon
+        d[3,4] = -1./L1mon
+        Rmon = Rm*(2*pi)**2/(8*pi*sin(thetam))*sqrt(det(f)/det((d*(s + t.T*f*t)**(-1)*d.T)**(-1) + g))
         R0 = R0*ki #1/ki monitor efficiency
         R0 = R0/Rmon
         #------ END OF ZHELUDEV's CORRECTION
+
         #----- Final error check
         if NP.imag.all() == 0:
             self.ERROR = None
@@ -720,6 +831,72 @@ Resolution Info:
 
     __repr__ = __str__
 
+    def run_mcstas(self, NMC, QE):
+        os.system('rm -rf /tmp/x')
+        cmd = '/data/software/ufit/mcstas/templateTAS.out -n%g' % (NMC * 1000)
+        cmd += ' --dir /tmp/x'
+        for p in self.par:
+            if p in ('qx', 'qy', 'qz', 'en', 'de', 'dqx', 'dqy', 'dqz', 'gh',
+                     'gk', 'gl', 'gmod', 'etas'):
+                continue
+            pn = p.upper()
+            if p == 'k':
+                pn = 'KFIX'
+            if p == 'kfix':
+                pn = 'FX'
+            cmd += ' %s=%s' % (pn, self.par[p])
+        cmd += ' L1=%s L2=%s L3=%s L4=%s' % (self.cfg[19]/100,
+                                             self.cfg[20]/100,
+                                             self.cfg[21]/100,
+                                             self.cfg[22]/100)
+        cmd += ' WM=%s HM=%s WA=%s HA=%s WD=%s HD=%s' % (self.cfg[14]/100,
+                                                         self.cfg[15]/100,
+                                                         self.cfg[17]/100,
+                                                         self.cfg[18]/100,
+                                                         self.cfg[11]/100,
+                                                         self.cfg[12]/100)
+        #if self.cfg[23] <= 0:
+        #    cmd += ' RMH=%s' % self.cfg[23]
+        #else:
+        #    cmd += ' RMH=%s' % (1/self.cfg[23]/100)
+        #if self.cfg[24] <= 0:
+        #    cmd += ' RMV=%s' % self.cfg[24]
+        #else:
+        #    cmd += ' RMV=%s' % (1/self.cfg[24]/100)
+        #if self.cfg[25] <= 0:
+        #    cmd += ' RAH=%s' % self.cfg[25]
+        #else:
+        #    cmd += ' RAH=%s' % (1/self.cfg[25]/100)
+        #if self.cfg[26] <= 0:
+        #    cmd += ' RAV=%s' % self.cfg[26]
+        #else:
+        #    cmd += ' RAV=%s' % (1/self.cfg[26]/100)
+        cmd += ' swidth=%s sheight=%s sthick=%s' % (self.cfg[7]/100,
+                                                    self.cfg[8]/100,
+                                                    self.cfg[9]/100)
+        cmd += ' QH=%s QK=%s QL=%s EN=%s' % tuple(QE)
+        print '[MCSTAS] running:', cmd
+        os.system(cmd)
+        print '[MCSTAS] finished'
+        try:
+            arr = loadtxt('/tmp/x/res.dat', ndmin=2)
+        except IOError:
+            return [], [], [], [], []
+        # some mcstas bug?!
+        if len(arr.T) == 4:
+            return [], [], [], [], []
+        kix, kiy, kiz, kfx, kfy, kfz, x, y, z, pi, pf = arr.T
+        # NOTE: cyclic shift of x, y, z here due to different coordinate system
+        # conventions in McStas and here:
+        # - McStas: z along beam, y upwards, x completes RHS
+        # - here:   x along beam, z upwards, y completes RHS
+        Q = array((kiz, kix, kiy)) - array((kfz, kfx, kfy))
+        w = 2.072*(kix**2 + kiy**2 + kiz**2 - kfx**2 - kfy**2 - kfz**2)
+        p = pi*pf/1e7
+        print '%d neutrons for MC calculation' % len(w)
+        Q = array([array(dot(self.unitc.cart2rluMat, q))[0] for q in Q.T]).T
+        return Q[0], Q[1], Q[2], w, p
+
     def calc_STrafo(self):
         """Calculates transformation matrix self.S, which transforms from the
         coordinate frame given in h, k, l (r.l.u.) and E(meV) to cartesian
@@ -733,9 +910,8 @@ Resolution Info:
         """
 
         #----- Work out transformations
-        a = self
-        A1 = matrix([a.par['ax'], a.par['ay'], a.par['az']]).transpose()
-        A2 = matrix([a.par['bx'], a.par['by'], a.par['bz']]).transpose()
+        A1 = matrix([self.par['ax'], self.par['ay'], self.par['az']]).transpose()
+        A2 = matrix([self.par['bx'], self.par['by'], self.par['bz']]).transpose()
         V1 = self.Q2c*A1
         V2 = self.Q2c*A2
         #----- Form unit vectors V1, V2, V3 in scattering plane
@@ -770,24 +946,38 @@ Resolution Info:
         #print V
 
         E = matrix(real(diag(E)))
-        sigma = zeros((1,4))[0]
+        sigma = zeros((1, 4))[0]
         sigma[0] = real(1./sqrt(E[0,0]))
         sigma[1] = real(1./sqrt(E[1,1]))
         sigma[2] = real(1./sqrt(E[2,2]))
         sigma[3] = real(1./sqrt(E[3,3]))
         self.b_mat = reshape(inv(V.transpose().transpose()), (1, 16))
-        #self.b_mat=reshape(inv((V.transpose())),(1,16))
+        self.sigma = sigma
+        #self.b_mat = reshape(inv((V.transpose())),(1,16))
         #print 'b_mat'
         #print self.b_mat
         return sigma
 
+    def setNPMatrix(self, m_matrix, (h, k, l)):
+        """Set fixed NP matrix for a given M matrix at Q point h,k,l."""
+        # see below for matrix meanings
+        TT = real(self.S*matrix([h, k, l]).transpose())
+        cos_theta = TT[0,0]/sqrt(dot(TT.transpose(),TT))
+        sin_theta = TT[1,0]/sqrt(dot(TT.transpose(),TT))
+
+        R = [[cos_theta, sin_theta, 0], [-sin_theta, cos_theta, 0], [0, 0, 1]]
+        T = matrix(zeros((4, 4)))
+        T[3, 3] = 1.
+        T[0:3, 0:3] = array(R*self.S).real
+        self.NP = T * m_matrix * T.transpose()
+
     def calcResEllipsoid(self, h, k, l, en):
         # set current Q vector and calculate corresponding resolution matrix
         # (sethklen calls self.calc_popovici() to calculate the matrix...
-        self.sethklen(h, k, l, en)
-
-        #   [R0,NP,vi,vf,Error]=feval(method,f,Qmag,p,mon_flag);
-        self.R0_corrected = real(self.R0/(sqrt(det(self.NP))/(2*pi)**2))
+        if not self.fixed_res:
+            self.sethklen(h, k, l, en)
+            #   [R0,NP,vi,vf,Error]=feval(method,f,Qmag,p,mon_flag);
+            self.R0_corrected = real(self.R0/(sqrt(det(self.NP))/(2*pi)**2))
         # corrected resolution volume as Monte Carlo integral is over a
         # normalised ellipsoid.
 
@@ -860,7 +1050,8 @@ Resolution Info:
         msg += 'NR' + ('Value   :  ').rjust(16) + 'Explanation\n'
         msg += '===============================\n'
         for i in range(len(self.cfg)):
-            msg += '%2i' % i + ('%1.3f   :  ' % self.cfg[i]).rjust(16) + '%s\n' % self.cfgnames[i]
+            msg += '%2i' % i + ('%1.3f   :  ' % self.cfg[i]).rjust(16) + \
+                '%s\n' % CFGDESC[i]
         print msg
 
     def resellipse(self):
@@ -1010,19 +1201,33 @@ def single_mc(NMC, sqw, fit_par, QE, b_mat, sigma, R0_corrected):
     qk = XMC[1] + QE[1]
     ql = XMC[2] + QE[2]
     w  = XMC[3] + QE[3]
-    #from numpy import ones
-    #qh = QE[0]*ones(NMC)
-    #qk = QE[1]*ones(NMC)
-    #ql = QE[2]*ones(NMC)
-    #w  = QE[3]*ones(NMC)
 
     # QE is provided to sqw function in case center of resolution
     # is needed for further calculations
-    mc_intens = sqw(qh, qk, ql, w, QE, sigma, *fit_par)
+    mc_intens = sqw(qh, qk, ql, w, QE, (b_mat, sigma), *fit_par)
     return R0_corrected * mc_intens.mean()
 
 pool = None
 
+class dummy_result(object):
+    def __init__(self, res):
+        self.res = res
+    def get(self):
+        return self.res
+
+
+def calc_MC_mcstas(x, fit_par, sqw, resmat, NMC):
+    """Trial with McStas."""
+    results = []
+    for QE in x:
+        qh, qk, ql, en, weights = resmat.run_mcstas(NMC, QE)
+        if qh == []:
+            mc_intens = 0
+        else:
+            mc_intens = (sqw(qh, qk, ql, en, QE, None, *fit_par) * weights).sum()
+        results.append(mc_intens)
+    return array(results)
+
 def calc_MC(x, fit_par, sqw, resmat, NMC, use_caching=True):
     """Calculates intensity of point in reciprocal space (qh,qk,ql,en) at takes
     into account the spectrometer resolution calculated by resolution class
@@ -1042,23 +1247,23 @@ def calc_MC(x, fit_par, sqw, resmat, NMC, use_caching=True):
                 print 'Scattering triangle will not close for point: ' \
                     'qh = %1.3f qk = %1.3f ql = %1.3f en = %1.3f' % tuple(QE)
                 print 'Attention: Intensity is therefore equal to zero at this point!'
-                # XXX will return a shorter list
+                results.append(dummy_result(0))
                 continue
             sigma = resmat.calcSigma()
             b_mat = resmat.b_mat[0:16]
             R0_corrected = resmat.R0_corrected
             resmat._cache[QE] = b_mat, sigma, R0_corrected
-        #results.append(single_mc(NMC, sqw, fit_par, QE, b_mat,
-        #                                            sigma, R0_corrected))
         results.append(pool.apply_async(single_mc, (NMC, sqw, fit_par, QE, b_mat,
                                                     sigma, R0_corrected)))
     return array([res.get() for res in results])
-    #return array(results)
 
 single_mc_cluster_code = '''
 from numpy import zeros, reshape
 from numpy.random import randn
 def single_mc(NMC, fit_par, QE, b_mat, sigma, R0_corrected):
+    if NMC == 0:
+        return 0.
+
     xp = zeros((4, NMC))
     xp[0,:] = sigma[0]*randn(NMC)
     xp[1,:] = sigma[1]*randn(NMC)
@@ -1074,7 +1279,7 @@ def single_mc(NMC, fit_par, QE, b_mat, sigma, R0_corrected):
 
     # QE is provided to sqw function in case center of resolution
     # is needed for further calculations
-    mc_intens = __sqw(qh, qk, ql, w, QE, sigma, *fit_par)
+    mc_intens = __sqw(qh, qk, ql, w, QE, (b_mat, sigma), *fit_par)
     return R0_corrected * mc_intens.mean()
 '''
 
@@ -1092,7 +1297,7 @@ def calc_MC_cluster(x, fit_par, sqwcode, sqwfunc, resmat, NMC, use_caching=True)
                 print 'Scattering triangle will not close for point: ' \
                     'qh = %1.3f qk = %1.3f ql = %1.3f en = %1.3f' % tuple(QE)
                 print 'Attention: Intensity is therefore equal to zero at this point!'
-                # XXX will return a shorter list
+                args.append((0, [], None, None, None, None))
                 continue
             sigma = resmat.calcSigma()
             b_mat = resmat.b_mat[0:16]
@@ -1103,44 +1308,6 @@ def calc_MC_cluster(x, fit_par, sqwcode, sqwfunc, resmat, NMC, use_caching=True)
     return array(cluster.run_cluster(code, 'single_mc', args))
 
 
-def demosqw(qh, qk, ql, en, QE, sigma, scaling):
-    """Simplest possible simulation of nuclear Bragg peak intensity plus
-    1-branch isotropic phonon dispersion to see something "realistic" in the
-    demo's virtual detector.
-    """
-    # get small q
-    qqh = divmod(qh, 1)[1]
-    if qqh > 0.5:
-        qqh -= 1
-    qqk = divmod(qk, 1)[1]
-    if qqk > 0.5:
-        qqk -= 1
-    qql = divmod(ql, 1)[1]
-    if qql > 0.5:
-        qql -= 1
-    qch = divmod(QE[0], 1)[1]
-    if qch > 0.5:
-        qch -= 1
-    qck = divmod(QE[1], 1)[1]
-    if qck > 0.5:
-        qck -= 1
-    qcl = divmod(QE[2], 1)[1]
-    if qcl > 0.5:
-        qcl -= 1
-    # Bragg intensity (will not work with MC integration, use center points and
-    # width given by resolution calculation, but we take the MC energy to get
-    # some randomness)
-    Ibr = 10000 * exp(-(qch**2/sigma[0]**2 + qck**2/sigma[1]**2 +
-                        qcl**2/sigma[2]**2 + en**2/sigma[3]**2))
-    # Phonon intensity (assume isotropic dispersion with Lorentz lineshape)
-    q = sqrt(qqh**2 + qqk**2 + qql**2)
-    omega = 10 * sin(q * pi)
-    gamma = 0.05
-    Iph = 10 / max(abs(en), 0.1) * gamma * (
-        1 / ((en - omega)**2 + gamma**2) + 1 / ((en + omega)**2 + gamma**2))
-    return 1e6 * scaling * (Ibr + Iph)
-
-
 def load_par(filename):
     pars = {}
     with open(filename) as f:
@@ -1160,3 +1327,105 @@ def load_cfg(filename):
             if line.strip():
                 pars.append(float(line.split()[0]))
     return pars
+
+
+def pylab_key_handler(event):
+    import pylab
+    if event.key == 'q':
+        pylab.close()
+
+def plot_resatpoint(cfg, par, resmat, fignum='Resolution calculation'):
+    import pylab
+
+    x, y, xslice, yslice, xxq, yxq, xxqslice, yxqslice, xyq, yyq, xyqslice, yyqslice = \
+        resmat.resellipse()
+
+    pylab.figure(fignum, figsize=(8.5, 6), dpi=120, facecolor='1.0')
+    pylab.clf()
+    pylab.rc('text', usetex=True)
+    pylab.rc('text.latex',
+             preamble='\\usepackage{amsmath}\\usepackage{helvet}\\usepackage{sfmath}')
+    pylab.subplots_adjust(left=0.11, bottom=0.08, right=0.97, top=0.81,
+                          wspace=0.25, hspace=0.27)
+    # register event handler to pylab
+    pylab.connect('key_press_event', pylab_key_handler)
+
+    pylab.subplot(221)
+    pylab.xlabel(r'Q$_x$ (\AA$^{-1}$)')
+    pylab.ylabel(r'Q$_y$ (\AA$^{-1}$)')
+    pylab.plot(x,y)
+    pylab.plot(xslice,yslice)
+
+    ax1 = pylab.gca()
+    text  = r"""\noindent\underline{Spectrometer Setup:}\newline
+\begin{tabular}{ll}
+d-spacings: & $d_M=%(dm)1.4f$\,\AA~~~$d_A=%(da)1.4f$\,\AA \\
+mosaic:     & $\eta_M=%(etam)3.1f'$~~~$\eta_S=%(etas)3.1f'$~~~$\eta_A=%(etaa)3.1f'$ \\
+s-sense:    & $s_M=%(sm)i$~~~$s_S=%(ss)i$~~~$s_A=%(sa)i$ \\
+$\alpha_{1\rightarrow4}$: & %(alpha1)i-Mono-%(alpha2)i-Sample-%(alpha3)i-Ana-%(alpha4)i (hor. coll.) \\
+$\beta_{1\rightarrow4}$:  & %(beta1)i-Mono-%(beta2)i-Sample-%(beta3)i-Ana-%(beta4)i   (vert. coll.) \\
+\end{tabular}
+""" % resmat.par
+    t1 = pylab.text(-0.25, 1.57, text.replace('\n', ''),
+                    horizontalalignment='left', verticalalignment='top',
+                    transform=ax1.transAxes)
+    t1.set_size(10)
+
+    pylab.subplot(222)
+    pylab.xlabel(r'Q$_x$ (\AA$^{-1}$)')
+    pylab.ylabel('Energy (meV)')
+    pylab.plot(xxq, yxq)
+    pylab.plot(xxqslice, yxqslice)
+
+    ax2 = pylab.gca()
+    text  = r"""\noindent\underline{Sample Parameters:}\newline
+\begin{tabular}{llllll}
+$a$ (\AA) & $b$ (\AA) & $c$ (\AA) & $\alpha$ ($^{\circ}$) & $\beta$ ($^{\circ}$) & $\gamma$ ($^{\circ}$) \\
+%(as)2.3f & %(bs)2.3f & %(cs)2.3f & %(aa)3.1f & %(bb)3.1f & %(cc)3.1f \\
+\end{tabular}\newline """ % resmat.par
+    if resmat.par['kfix'] == 1:
+        text += r'fixed incident energy $k_i=%2.4f$\,\AA$^{-1}$ ($\equiv %4.2f$\,meV)\newline ' % \
+            (resmat.par['k'], resmat.par['k']**2*2.07)
+    else:
+        text += r'fixed final energy $k_f=%2.4f$\,\AA$^{-1}$ ($\equiv %4.2f$\,meV)\newline ' % \
+            (resmat.par['k'], resmat.par['k']**2*2.07)
+    text += r'position: qh = %1.3f qk = %1.3f ql = %1.3f (r.l.u.) en = %2.3f (meV)\newline ' % \
+        (resmat.par['qx'], resmat.par['qy'], resmat.par['qz'], resmat.par['en'])
+    text += r'modulus of scattering vector $Q = %2.5f$\,\AA$^{-1}$' % resmat.q0
+    t2 = pylab.text(-0.25, 1.57, text.replace('\n', ' '),
+                    horizontalalignment='left', verticalalignment='top',
+                    transform=ax2.transAxes)
+    t2.set_size(10)
+
+    pylab.subplot(223)
+    pylab.xlabel(r'Q$_y$ (\AA$^{-1}$)')
+    pylab.ylabel(r'Energy (meV)')
+    pylab.plot(xyq,yyq)
+    pylab.plot(xyqslice,yyqslice)
+
+    pylab.subplot(224)
+    ax3 = pylab.gca()
+    pylab.axis('off')
+    pylab.rc('text', usetex=True)
+    text  = r'\noindent\underline{\textbf{Resolution Info:}}\newline ' + \
+        r'Resolution Volume: $R_0 = %7.5g$ (\AA$^{-3}$\,meV)\newline\newline ' % resmat.R0
+    mat = resmat.NP.tolist()
+    text += r'Resolution Matrix (in frame $Q_x$, $Q_y$, $Q_z$, $E$):\newline '
+    text += (r'$M = \left(\begin{array}{rrrr} %5.2f & %5.2f & %5.2f & %5.2f\\ ' \
+             r'%5.2f & %5.2f & %5.2f & %5.2f\\ %5.2f & %5.2f & %5.2f & %5.2f\\ ' \
+             r'%5.2f & %5.2f & %5.2f & %5.2f \end{array}\right)$\newline\newline\newline ') \
+             % (mat[0][0], mat[0][1], mat[0][2], mat[0][3],
+                mat[1][0], mat[1][1], mat[1][2], mat[1][3],
+                mat[2][0], mat[2][1], mat[2][2], mat[2][3],
+                mat[3][0], mat[3][1], mat[3][2], mat[3][3])
+    text += r'Bragg width:\newline '
+    text += r'\begin{tabular}{lllll} '
+    text += r'$Q_x$ (\AA$^{-1}$) & $Q_y$ (\AA$^{-1}$) & $Q_z$ (\AA$^{-1}$) & Vanadium & dE (meV) \\ '
+    bragw = tuple(resmat.calcBragg())
+    text += r'%1.5f & %1.5f & %1.5f & %1.5f & %1.5f \\ ' % bragw[:5]
+    text += r'\end{tabular}'
+    t3 = pylab.text(-0.13, 1.0, text,
+                    horizontalalignment='left', verticalalignment='top',
+                    transform=ax3.transAxes)
+    t3.set_size(10)
+    pylab.show()
